from typing import List, Union, Optional, Dict, Any

from ppa_api import images, exceptions

from .records import Image, ImageData, Task, TaskData


def get_all() -> List[Image]:
    return [ImageData(image.__dict__).__dict__ for image in images.get_all()]


def get_by_name(name: str) -> Union[Image, None]:
    try:
        return ImageData(**images.get_by_name(name=name).__dict__).__dict__
    except exceptions.NoSuchObject:
        return None


def get_by_id(image_id: Union[str, int]) -> Union[Image, None]:
    try:
        return ImageData(**images.get_by_id(image_id=image_id).__dict__).__dict__
    except exceptions.NoSuchObject:
        return None


def start_task_by_id(image_id: Union[str, int], payload: Optional[Dict[str, Any]] = None) -> Task:
    return TaskData(**images.start(image_id=image_id, payload=payload).__dict__).__dict__


def start_task_by_name(name: str, payload: Optional[Dict[str, Any]] = None) -> Task:
    return start_task_by_id(image_id=get_by_name(name)["id"], payload=payload)
