import datetime

from typing import List, Optional, TypedDict, Union

from datacaster.classes import CastDataClass

DateString = Optional[str]


def _date_string(date_object: datetime.datetime) -> str:
    return date_object.strftime("%H:%M:%S %d/%m/%Y")


def _optional_date_string(date_object: Union[datetime.datetime, None]) -> Union[datetime.datetime, None]:
    return _date_string(date_object) if date_object else None


class Image(TypedDict):
    id: int
    name: str
    description: str
    author: str
    tags: List[str]
    agent: str
    available: bool
    groups: int
    owner: Optional[str]
    updated_at: DateString


class ImageData(CastDataClass):
    __annotations__ = Image.__dict__["__annotations__"]
    __class_config__ = {
        "cast_functions": {"types": {DateString: _date_string}},
    }


class Task(TypedDict):
    id: int
    image_id: str
    uuid: str
    author: str
    username: str
    exit_code: Optional[int]
    defunct: bool
    started_at: DateString
    stopped_at: DateString
    cancelled_at: DateString
    cancelled_by: Optional[str]
    timed_out: bool
    state: str
    source: str


class TaskData(CastDataClass):
    __annotations__ = Task.__dict__["__annotations__"]
    __class_config__ = {
        "cast_functions": {"types": {DateString: _date_string}},
    }


class User(TypedDict):
    id: int
    name: str
    username: str
    email: str
    active: bool
    permissions: List[str]
    authenticated_at: Optional[DateString]


class UserData(CastDataClass):
    __annotations__ = User.__dict__["__annotations__"]
    __class_config__ = {
        "cast_functions": {"types": {DateString: _date_string, Optional[DateString]: _optional_date_string}},
    }
